package dev.lucasnlm.antimine.support

import android.content.Context
import android.widget.Toast
import dev.lucasnlm.antimine.R
import dev.lucasnlm.antimine.preferences.IPreferencesRepository
import dev.lucasnlm.external.IBillingManager
import dev.lucasnlm.external.model.PurchaseInfo
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.withContext

class IapHandler(
    private val context: Context,
    private val preferencesManager: IPreferencesRepository,
    private val billingManager: IBillingManager,
) {
    private val billingListener = MutableStateFlow(false)

    suspend fun start() {
        billingManager.listenPurchases().collect {
            if (it is PurchaseInfo.PurchaseResult) {
                onLockStatusChanged(it.unlockStatus)
            } else {
                showFailToConnectFeedback()
            }
        }
    }

    fun isEnabled() = billingManager.isEnabled()

    fun listenPurchase(): Flow<Boolean> = billingListener.asStateFlow()

    private fun onLockStatusChanged(status: Boolean) {
        billingListener.tryEmit(status)
        preferencesManager.setPremiumFeatures(status)
    }

    private suspend fun showFailToConnectFeedback() {
        withContext(Dispatchers.Main) {
            Toast.makeText(context, R.string.sign_in_failed, Toast.LENGTH_SHORT).show()
        }
    }
}
